@@ -1,5 +1,5 @@
 /* referral.c - LDBM backend referral handler */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/referral.c,v 1.1.2.2 2002/01/04 20:38:34 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/referral.c,v 1.1.2.3 2002/01/29 19:29:39 kurt Exp $ */
 /*
  * Copyright 2000-2002 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -38,6 +38,9 @@ ldbm_back_referrals(
 		return rc;
 	} 
 
+	/* grab giant lock for reading */
+	ldap_pvt_thread_rdwr_rlock(&li->li_giant_rwlock);
+
 	/* get entry with reader lock */
 	e = dn2entry_r( be, ndn, &matched );
 	if ( e == NULL ) {
@@ -58,6 +61,8 @@ ldbm_back_referrals(
 			cache_return_entry_r( &li->li_cache, matched );
 		}
 
+		ldap_pvt_thread_rdwr_runlock(&li->li_giant_rwlock);
+
 		if( refs != NULL ) {
 			/* send referrals */
 			send_ldap_result( conn, op, rc = LDAP_REFERRAL,
@@ -90,5 +95,7 @@ ldbm_back_referrals(
 	}
 
 	cache_return_entry_r( &li->li_cache, e );
+	ldap_pvt_thread_rdwr_runlock(&li->li_giant_rwlock);
+
 	return rc;
 }
